@@ -527,7 +527,9 @@ typedef struct
 {
   volatile gint ref_count;
   GSimpleAsyncResult *simple;
+  gboolean in_progress;
   gboolean completed;
+  gboolean failed;
 
   GVfsUDisks2Mount *mount;
 
@@ -541,6 +543,10 @@ typedef struct
 
   gulong mount_op_reply_handler_id;
   guint retry_unmount_timer_id;
+
+  GMountOperationResult reply_result;
+  gint reply_choice;
+  gboolean reply_set;
 } UnmountData;
 
 static UnmountData *
@@ -596,13 +602,14 @@ unmount_data_complete (UnmountData *data,
 {
   if (data->mount_operation &&
       !unmount_operation_is_eject (data->mount_operation))
-    gvfs_udisks2_unmount_notify_stop (data->mount_operation);
+    gvfs_udisks2_unmount_notify_stop (data->mount_operation, data->failed);
 
   if (complete_idle)
     g_simple_async_result_complete_in_idle (data->simple);
   else
     g_simple_async_result_complete (data->simple);
 
+  data->in_progress = FALSE;
   data->completed = TRUE;
   unmount_data_unref (data);
 }
@@ -620,6 +627,9 @@ on_retry_timer_cb (gpointer user_data)
   /* we're removing the timeout */
   data->retry_unmount_timer_id = 0;
 
+  if (data->completed || data->in_progress)
+    goto out;
+
   /* timeout expired => try again */
   unmount_do (data, FALSE);
 
@@ -627,6 +637,43 @@ on_retry_timer_cb (gpointer user_data)
   return FALSE; /* remove timeout */
 }
 
+static void
+mount_op_reply_handle (UnmountData *data)
+{
+  data->reply_set = FALSE;
+
+  if (data->reply_result == G_MOUNT_OPERATION_ABORTED ||
+      (data->reply_result == G_MOUNT_OPERATION_HANDLED &&
+       data->reply_choice == 1))
+    {
+      /* don't show an error dialog here */
+      g_simple_async_result_set_error (data->simple,
+                                       G_IO_ERROR,
+                                       G_IO_ERROR_FAILED_HANDLED,
+                                       "GMountOperation aborted (user should never see this "
+                                       "error since it is G_IO_ERROR_FAILED_HANDLED)");
+      data->failed = TRUE;
+      unmount_data_complete (data, TRUE);
+    }
+  else if (data->reply_result == G_MOUNT_OPERATION_HANDLED)
+    {
+      /* user chose force unmount => try again with force_unmount==TRUE */
+      unmount_do (data, TRUE);
+    }
+  else
+    {
+      /* result == G_MOUNT_OPERATION_UNHANDLED => GMountOperation instance doesn't
+       * support :show-processes signal
+       */
+      g_simple_async_result_set_error (data->simple,
+                                       G_IO_ERROR,
+                                       G_IO_ERROR_BUSY,
+                                       _("One or more programs are preventing the unmount operation."));
+      data->failed = TRUE;
+      unmount_data_complete (data, TRUE);
+    }
+}
+
 static void
 on_mount_op_reply (GMountOperation       *mount_operation,
                    GMountOperationResult result,
@@ -642,34 +689,11 @@ on_mount_op_reply (GMountOperation       *mount_operation,
   data->mount_op_reply_handler_id = 0;
 
   choice = g_mount_operation_get_choice (mount_operation);
-
-  if (result == G_MOUNT_OPERATION_ABORTED ||
-      (result == G_MOUNT_OPERATION_HANDLED && choice == 1))
-    {
-      /* don't show an error dialog here */
-      g_simple_async_result_set_error (data->simple,
-                                       G_IO_ERROR,
-                                       G_IO_ERROR_FAILED_HANDLED,
-                                       "GMountOperation aborted (user should never see this "
-                                       "error since it is G_IO_ERROR_FAILED_HANDLED)");
-      unmount_data_complete (data, TRUE);
-    }
-  else if (result == G_MOUNT_OPERATION_HANDLED)
-    {
-      /* user chose force unmount => try again with force_unmount==TRUE */
-      unmount_do (data, TRUE);
-    }
-  else
-    {
-      /* result == G_MOUNT_OPERATION_UNHANDLED => GMountOperation instance doesn't
-       * support :show-processes signal
-       */
-      g_simple_async_result_set_error (data->simple,
-                                       G_IO_ERROR,
-                                       G_IO_ERROR_BUSY,
-                                       _("One or more programs are preventing the unmount operation."));
-      unmount_data_complete (data, TRUE);
-    }
+  data->reply_result = result;
+  data->reply_choice = choice;
+  data->reply_set = TRUE;
+  if (!data->completed && !data->in_progress)
+    mount_op_reply_handle (data);
 }
 
 static void
@@ -788,6 +812,17 @@ unmount_show_busy (UnmountData  *data,
                    const gchar  *mount_point)
 {
   gchar *escaped_mount_point;
+
+  data->in_progress = FALSE;
+
+  /* We received an reply during an unmount operation which could not complete.
+   * Handle the reply now. */
+  if (data->reply_set)
+    {
+      mount_op_reply_handle (data);
+      return;
+    }
+
   escaped_mount_point = g_strescape (mount_point, NULL);
   gvfs_udisks2_utils_spawn (10, /* timeout in seconds */
                             data->cancellable,
@@ -811,7 +846,11 @@ lock_cb (GObject       *source_object,
   if (!udisks_encrypted_call_lock_finish (encrypted,
                                           res,
                                           &error))
-    g_simple_async_result_take_error (data->simple, error);
+    {
+      g_simple_async_result_take_error (data->simple, error);
+      data->failed = TRUE;
+    }
+
   unmount_data_complete (data, FALSE);
 }
 
@@ -838,6 +877,7 @@ unmount_cb (GObject       *source_object,
           goto out;
         }
       g_simple_async_result_take_error (data->simple, error);
+      data->failed = TRUE;
     }
   else
     {
@@ -879,6 +919,7 @@ umount_command_cb (GObject       *source_object,
                                         &error))
     {
       g_simple_async_result_take_error (data->simple, error);
+      data->failed = TRUE;
       unmount_data_complete (data, FALSE);
       goto out;
     }
@@ -902,6 +943,7 @@ umount_command_cb (GObject       *source_object,
                                    G_IO_ERROR,
                                    G_IO_ERROR_FAILED,
                                    "%s", standard_error);
+  data->failed = TRUE;
   unmount_data_complete (data, FALSE);
 
  out:
@@ -914,6 +956,8 @@ unmount_do (UnmountData *data,
 {
   GVariantBuilder builder;
 
+  data->in_progress = TRUE;
+
   if (data->mount_operation != NULL)
     gvfs_udisks2_unmount_notify_start (data->mount_operation, 
                                        G_MOUNT (data->mount), NULL,
@@ -1005,7 +1049,7 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
                                            G_IO_ERROR,
                                            G_IO_ERROR_FAILED,
                                            "No object for D-Bus interface");
-
+          data->failed = TRUE;
           unmount_data_complete (data, FALSE);
           goto out;
         }
@@ -1021,6 +1065,7 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
                                                G_IO_ERROR,
                                                G_IO_ERROR_FAILED,
                                                "No filesystem or encrypted interface on D-Bus object");
+              data->failed = TRUE;
               unmount_data_complete (data, FALSE);
               goto out;
             }
@@ -1037,6 +1082,7 @@ gvfs_udisks2_mount_unmount_with_operation (GMount              *_mount,
                                                    G_IO_ERROR,
                                                    G_IO_ERROR_FAILED,
                                                    "No filesystem interface on D-Bus object for cleartext device");
+                  data->failed = TRUE;
                   unmount_data_complete (data, FALSE);
                   goto out;
                 }
@@ -1240,7 +1286,12 @@ gvfs_udisks2_mount_guess_content_type_sync (GMount        *_mount,
       if (gudev_device != NULL)
         {
           /* Check if its bootable */
-          if (g_udev_device_get_property_as_boolean (gudev_device, "OSINFO_BOOTABLE"))
+          const gchar *boot_sys_id;
+
+          boot_sys_id = g_udev_device_get_property (gudev_device,
+                                                    "ID_FS_BOOT_SYSTEM_ID");
+          if (boot_sys_id != NULL ||
+              g_udev_device_get_property_as_boolean (gudev_device, "OSINFO_BOOTABLE"))
             g_ptr_array_add (p, g_strdup ("x-content/bootable-media"));
 
           /* Check for media player */
